<?php

class JsonDataCollector
{

    // Job properties data map
    protected $properties = [
        'livas-id'      => 'id',
        'valid'         => 'galioja',
        'post_date'     => 'data',
        'post_modified' => 'atnaujinta',
        'title'         => 'pareigos',
        'post_content'  => 'aprasymas',
//        'email'         => 'email',
//        'work_time'     => 'darbo_laikas',
//        'contact'       => 'kontaktai',
    ];

    // Taxonomies data map
    protected $terms = [
        'city' => [
            'slug' => 'miestas_id',
            'name' => 'regionas'
        ],
        'field' => [
            'slug' => 'kategorija',
            'name' => 'sritis'
        ],
        'company' => [
            'name' => 'imone'
        ],
        'type' => [
            'slug' => 'type',
            'name' => 'type'
        ]
//        'tag'
//        'position',
    ];

    // Custom posts data map
    protected $posts = [
        'division' => [
            'title' => 'padalinys_name',
            'slug' => 'padalinys'
        ]
    ];

    // Language map
    protected $lang = 'kalba';

    // Relations mapping
    protected $relationsMap = [
        'division-id' => 'division',
    ];

    // Source data
    protected $sourceData;

    protected $data;

    public function __construct($source)
    {
        $this->setData($source);
        $this->data = $this->transformData();
    }

    /**
     *  Set data source:
     *  get data from source and transform into array
     *
     * @param $source
     */
    protected function setData($source)
    {
        $this->sourceData = $this->loadData($source);
    }

    /**
     *  Parse xml data
     *
     * @param $source
     *
     * @return SimpleXMLElement
     */
    protected function loadData($source)
    {
        $context = stream_context_create([
            'http' => [
                'header' => 'Accept: application/json',
            ],
            'ssl'  => [
                'verify_peer'      => false,
                'verify_peer_name' => false,
            ],
        ]);
        $json = file_get_contents($source, false, $context);

        return json_decode($json);
    }

    /**
     *  Get data
     *
     * @return mixed
     */
    public function getData()
    {
        return $this->data;
    }

    protected function transformData()
    {
        $result = [];
        foreach ($this->sourceData as $ad) {
            $item = [];

            $item['properties'] = $this->getProperties($ad);
            $item['lang'] = $ad->{$this->lang};
            $item['terms'] = $this->getTerms($ad);
            $item['posts'] = $this->getPosts($ad);
            $item['relations'] = $this->relationsMap;

//            foreach ($this->dataMap as $mapKey => $dataKey) {
//                $dataValue = $ad->{$dataKey};
//                $itemValue = $this->getValue($dataValue);
//                $postValue = $this->getValue($dataValue);
//                $item[$mapKey] = $itemValue;
//
////                if (array_key_exists($mapKey, $this->relationsMap)) {
////                    $item['related_items'][$mapKey] = $this->relationsMap[$mapKey];
////                }
//            }
//
//            foreach ($this->relationsMap as $keyItem => $valueItem) {
//                if (array_key_exists($keyItem, $item) && array_key_exists($valueItem, $item)) {
//                    $item[$valueItem] = [$valueItem . '_' . $item[$keyItem] => $item[$valueItem]];
//                    unset($item[$keyItem]);
//                }
//            }

            $result[] = $item;
        };

        return $result;
    }

    private function getProperties($ad)
    {
        $result = [];

        foreach ($this->properties as $propertyKey => $propertyValue) {
            if (isset($ad->{$propertyValue}))
            $dataValue = $ad->{$propertyValue};
            $result[$propertyKey] = $this->getValue($dataValue);
        }

        return $result;
    }

    private function getTerms($ad)
    {
        $result = [];

        foreach ($this->terms as $termKey => $termProperties) {
            foreach ($termProperties as $propertyKey => $propertyValue) {
                if (isset($ad->{$propertyValue})) {
                    $dataValue = $ad->{$propertyValue};
                    $result[$termKey][$propertyKey] = $this->getValue($dataValue);
                }
            }
        }

        return $result;
    }

    private function getPosts($ad)
    {
        $result = [];

        foreach ($this->posts as $postKey => $postProperties) {
            foreach ($postProperties as $propertyKey => $propertyValue) {
                if (isset($ad->{$propertyValue})) {
                    $dataValue = $ad->{$propertyValue};
                    $result[$postKey][$propertyKey] = $this->getValue($dataValue);
                }
            }
        }

        return $result;
    }

    private function getValue($value)
    {
        return (is_array($value)) ? $value : html_entity_decode($value);
    }
}